package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.Collection;
import java.util.List;

import org.w3c.dom.Document;

import de.fhhannover.inform.trust.ifmapj.identifier.Identifier;

/**
 * Implementation of interface {@link RequestFactory}
 * 
 * @author aw
 *
 */
public class RequestFactoryImpl implements RequestFactoryInternal, RequestFactory {

	@Override
	public PublishRequest createPublishReq() {
		return new PublishRequestImpl();
	}
	
	@Override
	public PublishRequest createPublishReq(PublishElement el) {
		if (el == null)
			throw new NullPointerException("el is not allowed to be null");
		
		PublishRequest ret = createPublishReq();
		ret.addPublishElement(el);
		return ret;
	}

	@Override
	public PublishRequest createPublishReq(Collection<PublishElement> list) {
		if (list == null)
			throw new NullPointerException("list is not allowed to be null");

		PublishRequest ret = createPublishReq();
		for (PublishElement el : list)
			ret.addPublishElement(el);
		
		return ret;
	}

	@Override
	public NewSessionRequest createNewSessionReq() {
		return new NewSessionRequestImpl();
	}

	@Override
	public EndSessionRequest createEndSessionReq() {
		return new EndSessionRequestImpl();
	}

	@Override
	public RenewSessionRequest createRenewSessionReq() {
		return new RenewSessionRequestImpl();
	}

	@Override
	public PurgePublisherRequest createPurgePublisherReq() {
		return new PurgePublisherRequestImpl();
	}

	@Override
	public PollRequest createPollReq() {
		return new PollRequestImpl();
	}


	@Override
	public SubscribeRequest createSubscribeReq() {
		return new SubscribeRequestImpl();
	}
	
	@Override
	public SubscribeRequest createSubscribeReq(SubscribeElement el) {
		if (el == null)
			throw new NullPointerException("el is not allowed to be null");
		
		SubscribeRequest ret = createSubscribeReq();
		ret.addSubscribeElement(el);
		return ret;
	}

	@Override
	public SubscribeRequest createSubscribeReq(List<SubscribeElement> list) {
		if (list == null)
			throw new NullPointerException("list is not allowed to be null");
		
		SubscribeRequest ret = createSubscribeReq();
		for (SubscribeElement el : list)
			ret.addSubscribeElement(el);
		
		return ret;
	}

	@Override
	public SearchRequest createSearchRequest() {
		return new SearchRequestImpl(new SearchHolderImpl());
	}

	@Override
	public SearchRequest createSearchRequest(String matchLinks, Integer maxDepth,
			String termIdents, Integer maxSize, String resultFilter, Identifier start) {
		SearchRequest sr = createSearchRequest();
		fillSearchHolder(sr, matchLinks, maxDepth, termIdents, maxSize, resultFilter, start);
		return sr;
	}

	private void fillSearchHolder(SearchHolder holder, String matchLinks,
			Integer maxDepth, String termIdents, Integer maxSize,
			String resultFilter, Identifier start) {
		holder.setMatchLinksFilter(matchLinks);
		holder.setMaxDepth(maxDepth);
		holder.setTerminalIdentifierTypes(termIdents);
		holder.setMaxSize(maxSize);
		holder.setResultFilter(resultFilter);
		holder.setStartIdentifier(start);
	}

	@Override
	public PublishUpdate createPublishUpdate() {
		return new PublishUpdateImpl();
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Document md) {
		return createPublishUpdate(i1, md, MetadataLifetime.session);
	}
	
	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Document md,
			MetadataLifetime lifetime) {
		return createPublishUpdate(i1, null, md, lifetime);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1,
			Collection<Document> mdlist, MetadataLifetime lifetime) {
		return createPublishUpdate(i1, null, mdlist, lifetime);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Identifier i2,
			Document md) {
		return createPublishUpdate(i1, i2, md, MetadataLifetime.session);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Identifier i2,
			Document md, MetadataLifetime lifetime) {
		if (md == null)
			throw new NullPointerException("md not allowed to be null");
		return createPublishUpdate(i1, i2, md, null, lifetime);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1,
			Collection<Document> mdlist) {
		return createPublishUpdate(i1, null, mdlist);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Identifier i2,
			Collection<Document> mdlist) {
		return createPublishUpdate(i1, i2, mdlist, MetadataLifetime.session);
	}

	@Override
	public PublishUpdate createPublishUpdate(Identifier i1, Identifier i2,
			Collection<Document> mdlist, MetadataLifetime lifetime) {
		if (mdlist == null)
			throw new NullPointerException("mdlist not allowed to be null");
		return createPublishUpdate(i1, i2, null, mdlist, lifetime);
	}
	
	private PublishUpdate createPublishUpdate(Identifier i1, Identifier i2,
			Document mdelement, Collection<Document> mdlist,
			MetadataLifetime lifetime) {
		PublishUpdate pu = createPublishUpdate();
		fillMetadataHolder(pu, i1, i2, mdelement, mdlist);
		pu.setLifeTime(lifetime);
		return pu;
	}
	
	@Override
	public PublishNotify createPublishNotify() {
		return new PublishNotifyImpl();
	}

	@Override
	public PublishNotify createPublishNotify(Identifier i1, Document md) {
		return createPublishNotify(i1, null, md);
	}


	@Override
	public PublishNotify createPublishNotify(Identifier i1,
			Collection<Document> md) {
		return createPublishNotify(i1, null, md);
	}

	@Override
	public PublishNotify createPublishNotify(Identifier i1, Identifier i2,
			Document md) {
		if (md == null)
			throw new NullPointerException("md is null");
		
		return createPublishNotify(i1, i2, md, null);
	}

	@Override
	public PublishNotify createPublishNotify(Identifier i1, Identifier i2,
			Collection<Document> mdlist) {
		if (mdlist == null)
			throw new NullPointerException("mdlist is null");
		
		return createPublishNotify(i1, i2, null, mdlist);
	}
	
	private PublishNotify createPublishNotify(Identifier i1, Identifier i2,
			Document mdelement, Collection<Document> mdlist) {
		PublishNotify pn = createPublishNotify();
		fillMetadataHolder(pn, i1, i2, mdelement, mdlist);
		return pn;
	}

	private void fillMetadataHolder(MetadataHolder holder, Identifier i1,
			Identifier i2, Document mdelement, Collection<Document> mdlist) {
		fillIdentifierHolder(holder, i1, i2);
		
		if (mdlist != null) {
			for (Document md : mdlist)
				holder.addMetadata(md);
		}
		
		if (mdelement != null)
			holder.addMetadata(mdelement);
	}

	private void fillIdentifierHolder(IdentifierHolder holder, Identifier i1,
			Identifier i2) {
		holder.setIdentifier1(i1);
		holder.setIdentifier2(i2);
	}

	@Override
	public PublishDelete createPublishDelete() {
		return new PublishDeleteImpl();
	}

	@Override
	public PublishDelete createPublishDelete(Identifier i1) {
		return createPublishDelete(i1, (String)null);
	}

	@Override
	public PublishDelete createPublishDelete(Identifier i1, Identifier i2) {
		return createPublishDelete(i1, i2, null);
	}

	@Override
	public PublishDelete createPublishDelete(Identifier i1, String filter) {
		return createPublishDelete(i1, null, filter);
	}

	@Override
	public PublishDelete createPublishDelete(Identifier i1, Identifier i2,
			String filter) {
		PublishDelete pd = createPublishDelete();
		pd.setIdentifier1(i1);
		pd.setIdentifier2(i2);
		pd.setFilter(filter);
		return pd;
	}

	@Override
	public SubscribeUpdate createSubscribeUpdate() {
		return new SubscribeUpdateImpl(new SearchHolderImpl());
	}
	
	@Override
	public SubscribeUpdate createSubscribeUpdate(String name,
			String matchLinks, Integer maxDepth, String termIdents,
			Integer maxSize, String resultFilter, Identifier start) {
		SubscribeUpdate su = createSubscribeUpdate();
		fillSearchHolder(su, matchLinks, maxDepth, termIdents, maxSize, resultFilter, start);
		su.setName(name);
		return su;
	}

	@Override
	public SubscribeDelete createSubscribeDelete() {
		return new SubscribeDeleteImpl();
	}

	@Override
	public SubscribeDelete createSubscribeDelete(String name) {
		SubscribeDelete sd = createSubscribeDelete();
		sd.setName(name);
		return sd;
	}


}
