package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.Collection;

import util.Pair;

import de.fhhannover.inform.trust.ifmapj.identifier.Identifier;

/**
 * Implement by delegating to {@link SearchHolderImpl}.
 * 
 * @author aw
 *
 */
class SearchRequestImpl extends RequestImpl implements SearchRequest {
	
	
	private final SearchHolder mSearchHolder;

	SearchRequestImpl(SearchHolder searchHolder) {
		if (searchHolder == null)
			throw new NullPointerException();
		
		mSearchHolder = searchHolder;
	}

	@Override
	public void addNamespaceDeclaration(String prefix, String uri) {
		mSearchHolder.addNamespaceDeclaration(prefix, uri);
	}

	@Override
	public Collection<Pair<String, String>> getNamespaceDeclarations() {
		return mSearchHolder.getNamespaceDeclarations();
	}

	@Override
	public Identifier getStartIdentifier() {
		return mSearchHolder.getStartIdentifier();
	}

	@Override
	public void setStartIdentifier(Identifier startIdentifier) {
		mSearchHolder.setStartIdentifier(startIdentifier);
	}

	@Override
	public String getMatchLinksFilter() {
		return mSearchHolder.getMatchLinksFilter();
	}

	@Override
	public void setMatchLinksFilter(String matchLinksFilter) {
		mSearchHolder.setMatchLinksFilter(matchLinksFilter);
	}

	@Override
	public Integer getMaxDepth() {
		return mSearchHolder.getMaxDepth();
	}

	@Override
	public void setMaxDepth(Integer maxDepth) {
		mSearchHolder.setMaxDepth(maxDepth);
	}

	@Override
	public String getTerminalIdentifierTypes() {
		return mSearchHolder.getTerminalIdentifierTypes();
	}

	@Override
	public void setTerminalIdentifierTypes(String terminalIdentifierTypes) {
		mSearchHolder.setTerminalIdentifierTypes(terminalIdentifierTypes);
	}

	@Override
	public Integer getMaxSize() {
		return mSearchHolder.getMaxSize();
	}

	@Override
	public void setMaxSize(Integer maxSize) {
		mSearchHolder.setMaxSize(maxSize);
	}

	@Override
	public String getResultFilter() {
		return mSearchHolder.getResultFilter();
	}

	@Override
	public void setResultFilter(String resultFilter) {
		mSearchHolder.setResultFilter(resultFilter);
	}
}
