package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

/**
 * A {@link SearchResult} simply contains a list of {@link ResultItem} objects.
 * Optionally it may contain a name in case this {@link SearchResult} is
 * contained in a {@link PollResult};
 * 
 * @author aw
 *
 */
class SearchResultImpl implements SearchResult {
	
	private final Collection<ResultItem> mResultItems;
	
	private final String mName;

	SearchResultImpl(Collection<ResultItem> ritems, String name) {
		
		if (ritems == null)
			throw new NullPointerException("ResultItem list is null");
		
		mResultItems = new LinkedList<ResultItem>(ritems);
		mName = name;
	}
	
	SearchResultImpl(Collection<ResultItem> ritems) {
		this(ritems, null);
	}

	@Override
	public Collection<ResultItem> getResultItems() {
		return Collections.unmodifiableCollection(mResultItems);
	}

	@Override
	public String getName() {
		return mName;
	}
	
	@Override
	public String toString() {
		int i = 0;
		StringBuilder sb = new StringBuilder("sr{");
		if (mName != null) {
			sb.append(mName);
			sb.append(", ");
		}
		for (ResultItem ri : mResultItems) {
			i++;
			sb.append(ri.toString());
			if (i != mResultItems.size())
				sb.append(", ");
		}
		sb.append("}");
		return sb.toString();
	}
}
